searchData={"items":[{"type":"module","title":"ACO.Ant","doc":"","ref":"ACO.Ant.html"},{"type":"function","title":"ACO.Ant.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ACO.Ant.html#child_spec/1"},{"type":"module","title":"ACO.Colony","doc":"","ref":"ACO.Colony.html"},{"type":"module","title":"Bia","doc":"Documentation for `Bia`, a bio-inspired algorithms library written in Elixir.","ref":"Bia.html"},{"type":"module","title":"PSO","doc":"Implementation of Particle Swarm Optimization in Elixir.\n\nVelocity is updated with\n\n$$\nv_{i,d} \\leftarrow \\omega v_{i,d} + \\phi_p r_p (p_{i,d}-x_{i,d}) + \\phi_g r_g (g_d-x_{i,d})\n$$","ref":"PSO.html"},{"type":"function","title":"PSO.new/1","doc":"Creates a new instance of a PSO.","ref":"PSO.html#new/1"},{"type":"function","title":"Options - PSO.new/1","doc":"* `:population_size` (`t:pos_integer/0`) - The number of particles to be used in the optimization. The default value is `10`.\n\n* `:num_iterations` (`t:non_neg_integer/0`) - The number of iterations to be done in the optimization. The default value is `100`.\n\n* `:dimensions` (`t:pos_integer/0`) - The dimensions of the search space. The default value is `2`.\n\n* `:bound_up` (`t:float/0`) - The upper boundary of the search space. The default value is `5.12`.\n\n* `:bound_down` (`t:float/0`) - The lower boundary of the search space. The default value is `-5.12`.\n\n* `:inertia` (`t:float/0`) - The inertia each particle carries each move. Should be smaller than 1. The default value is `0.6`.\n\n* `:coeff_p` (`t:float/0`) - The cognitive coefficient. The default value is `1.0`.\n\n* `:coeff_g` (`t:float/0`) - The social coefficient. The default value is `3.0`.\n\n* `:fun` (function of arity 1) - The function to optimize. The default value is `&Nx.sum/1`.\n\n* `:callback` (function of arity 1) - A function to be called after each iteration. The default value is `&PSO.callback/1`.\n\n* `:widget` (`t:term/0`) - A widget for getting the data visualized. The default value is `nil`.","ref":"PSO.html#new/1-options"},{"type":"function","title":"Return Values - PSO.new/1","doc":"The function returns a tuple of two values:\n\n  * `supervisor` - the pid of the created Swarm supervisor.\n\n  * `opts` - the parameters of the initialized Swarm.","ref":"PSO.html#new/1-return-values"},{"type":"function","title":"PSO.run/1","doc":"Runs a given instance of an initialized Swarm with pid `supervisor` and options `opts`.","ref":"PSO.html#run/1"},{"type":"function","title":"Return Values - PSO.run/1","doc":"The function returns a map with the following:\n\n  * `best_position` - the position of the best result found.\n\n  * `best` - the best result found.","ref":"PSO.html#run/1-return-values"},{"type":"type","title":"PSO.config/0","doc":"","ref":"PSO.html#t:config/0"},{"type":"type","title":"PSO.results/0","doc":"","ref":"PSO.html#t:results/0"},{"type":"type","title":"PSO.supervisor/0","doc":"","ref":"PSO.html#t:supervisor/0"},{"type":"extras","title":"Bia","doc":"# Bia\n\nBio-inspired algorithms in Elixir.\n\nSupported heuristics:\n\n- [x] Particle Swarm Optimization (PSO)\n- [ ] Ant Colony Optimization (ACO)","ref":"readme.html"},{"type":"extras","title":"PSO example","doc":"# PSO example\n\n```elixir\nMix.install([\n  {:bia, github: \"matiascr/bia\"},\n  {:kino, \"~> 0.10.0\"},\n  {:kino_vega_lite, \"~> 0.1.10\"},\n  {:vega_lite, \"~> 0.1.8\"},\n  {:explorer, \"~> 0.7.1\"}\n])\n```","ref":"pso_example.html"},{"type":"extras","title":"Setup - PSO example","doc":"We start by defining aliases for the libraries we'll use for visualizing the PSO\n\n```elixir\nalias VegaLite, as: Vl\nrequire Kino.VegaLite, as: KVl\n```\n\nNow, we define a Visualizer module that contains a callback\n\n```elixir\ndefmodule Visualizer do\n  def callback(args) do\n    widget = args[:opts][:widget]\n\n    widget\n    |> KVl.clear()\n\n    widget\n    |> KVl.push_many(\n      Enum.map(args[:particles], &GenServer.call(&1, :get_position))\n      |> Nx.stack()\n      |> Nx.to_list()\n      |> Enum.map(fn [x, y] -> %{x: x, y: y} end)\n    )\n\n    Process.sleep(150)\n  end\nend\n```\n\nand create it with the parameters we want for the graph\n\n```elixir\nbound_up = 10.0\nbound_down = -10.0\n\nwidget = fn ->\n  Vl.new(width: 400, height: 400)\n  |> Vl.mark(:circle)\n  |> Vl.encode_field(:x, \"x\", type: :quantitative, scale: [domain: [bound_down, bound_up]])\n  |> Vl.encode_field(:y, \"y\", type: :quantitative, scale: [domain: [bound_down, bound_up]])\n  |> KVl.new()\n  |> Kino.render()\nend\n```\n\nWe add some functions for the PSO to optimize\n\n```elixir\ndefmodule OptimizationFunctions do\n  import Nx.Defn\n\n  defn unimodal(t) do\n    0.26 * (t[0] ** 2 + t[1] ** 2) - 0.48 * t[0] * t[1]\n  end\n\n  defn inverted_pyramid(t) do\n    Nx.abs(t[0]) + Nx.abs(t[1])\n  end\nend\n```","ref":"pso_example.html#setup"},{"type":"extras","title":"Running the PSO - PSO example","doc":"Finally, we run the PSO algorithm.\n\nIn the first step, we add the parameters of the heuristic, along with other optional parameters for in the callback and widget parameters. These will allow us to inject actions that will be performed after each iteration of the PSO. In this case, we have created a graph, and it will be updated each iteration with the position of the particles.\n\n```elixir\nPSO.new(\n  population_size: 50,\n  num_iterations: 30,\n  bound_up: bound_up,\n  bound_down: bound_down,\n  inertia: 0.1,\n  coeff_p: 2.0,\n  coeff_g: 1.0,\n  callback: &Visualizer.callback/1,\n  widget: widget.(),\n  fun: &OptimizationFunctions.inverted_pyramid/1\n)\n|> PSO.run()\n```","ref":"pso_example.html#running-the-pso"}],"content_type":"text/markdown"}